@@ -9,6 +9,7 @@ __license__   = "MIT"
 
 import saga.utils.pty_shell
 
+import saga.url as surl
 import saga.adaptors.base
 import saga.adaptors.cpi.job
 
@@ -18,7 +19,6 @@ from transferdirectives import TransferDirectives
 import re
 import os
 import time
-from copy import deepcopy
 from urlparse import parse_qs
 from tempfile import NamedTemporaryFile
 
@@ -352,7 +352,7 @@ class CondorJobService (saga.adaptors.cpi.job.Service):
         self.query_options = dict()
 
         rm_scheme = rm_url.scheme
-        pty_url   = deepcopy(rm_url)
+        pty_url   = surl.Url (rm_url)
 
         # this adaptor supports options that can be passed via the
         # 'query' component of the job service URL.
@@ -506,7 +506,7 @@ class CondorJobService (saga.adaptors.cpi.job.Service):
             # we don't want the 'query' part of the URL to be part of the ID,
             # simply because it can get terribly long (and ugly). to get rid
             # of it, we clone the URL and set the query part to None.
-            rm_clone = deepcopy(self.rm)
+            rm_clone = surl.Url (self.rm)
             rm_clone.query = ""
             rm_clone.path = ""
 
@@ -546,8 +546,8 @@ class CondorJobService (saga.adaptors.cpi.job.Service):
         rm, pid = self._adaptor.parse_id(job_id)
 
         # run the Condor 'condor_q' command to get some infos about our job
-        ret, out, _ = self.shell.run_sync("%s -long %s | \
-            egrep '(JobStatus)|(ExitStatus)|(CompletionDate)'" \
+        ret, out, _ = self.shell.run_sync("unset GREP_OPTIONS; %s -long %s | \
+            grep -E '(JobStatus)|(ExitStatus)|(CompletionDate)'" \
             % (self._commands['condor_q']['path'], pid))
         if ret != 0:
             message = "Couldn't reconnect to job '%s': %s" % (job_id, out)
@@ -659,14 +659,23 @@ class CondorJobService (saga.adaptors.cpi.job.Service):
             return prev_info
 
         # curr. info will contain the new job info collect. it starts off
-        # as a copy of prev_info
-        curr_info = deepcopy(prev_info)
+        # as a copy of prev_info (don't use deepcopy because there is an API 
+        # object in the dict -> recursion)
+        curr_info = dict()
+        curr_info['job_id'     ] = prev_info.get ('job_id'     )
+        curr_info['state'      ] = prev_info.get ('state'      )
+        curr_info['exec_hosts' ] = prev_info.get ('exec_hosts' )
+        curr_info['returncode' ] = prev_info.get ('returncode' )
+        curr_info['create_time'] = prev_info.get ('create_time')
+        curr_info['start_time' ] = prev_info.get ('start_time' )
+        curr_info['end_time'   ] = prev_info.get ('end_time'   )
+        curr_info['gone'       ] = prev_info.get ('gone'       )
 
         rm, pid = self._adaptor.parse_id(job_id)
 
         # run the Condor 'condor_q' command to get some infos about our job
-        ret, out, _ = self.shell.run_sync("%s -long %s | \
-            egrep '(JobStatus)|(ExitStatus)|(CompletionDate)'" \
+        ret, out, _ = self.shell.run_sync("unset GREP_OPTIONS; %s -long %s | \
+            grep -E '(JobStatus)|(ExitStatus)|(CompletionDate)'" \
             % (self._commands['condor_q']['path'], pid))
 
         if ret != 0:
@@ -674,8 +683,8 @@ class CondorJobService (saga.adaptors.cpi.job.Service):
 
                 # run the Condor 'condor_history' command to get info about 
                 # finished jobs
-                ret, out, _ = self.shell.run_sync("%s -long -match 1 %s | \
-                    egrep '(ExitCode)|(TransferOutput)|(CompletionDate)|(JobCurrentStartDate)|(QDate)|(Err)|(Out)'" \
+                ret, out, _ = self.shell.run_sync("unset GREP_OPTIONS; %s -long -match 1 %s | \
+                    grep -E '(ExitCode)|(TransferOutput)|(CompletionDate)|(JobCurrentStartDate)|(QDate)|(Err)|(Out)'" \
                     % (self._commands['condor_history']['path'], pid))
                 
                 if ret != 0:
@@ -965,7 +974,7 @@ class CondorJobService (saga.adaptors.cpi.job.Service):
         """
         ids = []
 
-        ret, out, _ = self.shell.run_sync("%s | grep `whoami`"\
+        ret, out, _ = self.shell.run_sync("unset GREP_OPTIONS; %s | grep `whoami`"\
             % self._commands['condor_q']['path'])
 
         if ret != 0 and len(out) > 0:
@@ -979,7 +988,7 @@ class CondorJobService (saga.adaptors.cpi.job.Service):
                 # 112059.svc.uc.futuregrid testjob oweidner 0 Q batch
                 # 112061.svc.uc.futuregrid testjob oweidner 0 Q batch
                 if len(line.split()) > 1:
-                    rm_clone = deepcopy(self.rm)
+                    rm_clone = surl.Url (self.rm)
                     rm_clone.query = ""
                     rm_clone.path = ""
 
