@@ -22,6 +22,26 @@ SYNC_CALL  = saga.adaptors.cpi.decorators.SYNC_CALL
 ASYNC_CALL = saga.adaptors.cpi.decorators.ASYNC_CALL
 
 
+# --------------------------------------------------------------------
+#
+# strip white space from a string, and hex-decode the remaining characters.
+# This must be applied to stdout/stderr data returned from the shell wrapper.
+#
+def _decode (data) :
+
+    elem = ""
+    code = ""
+    
+    for c in data :
+    
+        if  c in ['0','1','2','3','4','5','6','7','8','9','a','b','c','d','e','f'] :
+            code += c
+        elif c not in [' ', '\n'] :
+            raise saga.BadParameter("Cannot decode '%s' in data (%s)" % (c, data))
+    
+    return code.decode ('hex')
+
+
 # --------------------------------------------------------------------
 # the adaptor name
 #
@@ -557,8 +577,11 @@ class ShellJobService (saga.adaptors.cpi.job.Service) :
 
         ret, out, _ = self.shell.run_sync ("RESULT %s\n" % pid)
         if  ret != 0 :
-            raise saga.NoSuccess ("failed to get exit code for '%s': (%s)(%s)" \
+          # raise saga.NoSuccess ("failed to get exit code for '%s': (%s)(%s)" \
+          #                    % (id, ret, out))
+            self._logger.warning ("failed to get exit code for '%s': (%s)(%s)" \
                                % (id, ret, out))
+            return None
 
         lines = filter (None, out.split ("\n"))
         self._logger.debug (lines)
@@ -568,10 +591,14 @@ class ShellJobService (saga.adaptors.cpi.job.Service) :
             del (lines[0])
 
         if  len (lines) != 2 :
-            raise saga.NoSuccess ("failed to get exit code for '%s': (%s)" % (id, lines))
+          # raise saga.NoSuccess ("failed to get exit code for '%s': (%s)" % (id, lines))
+            self._logger.warning ("failed to get exit code for '%s': (%s)" % (id, lines))
+            return None
 
         if lines[0] != "OK" :
-            raise saga.NoSuccess ("failed to get exit code for '%s' (%s)" % (id, lines))
+          # raise saga.NoSuccess ("failed to get exit code for '%s' (%s)" % (id, lines))
+            self._logger.warning ("failed to get exit code for '%s' (%s)" % (id, lines))
+            return None
 
         exit_code = lines[1].strip ()
 
@@ -708,12 +735,16 @@ class ShellJobService (saga.adaptors.cpi.job.Service) :
         self._ids = []
 
         for line in lines :
+
             try :
-                pid    = int(line.strip ())
-                job_id = "[%s]-[%s]" % (self.rm, pid)
-                self._ids.append (job_id)
+                pid = int(line.strip ())
+
             except Exception as e:
-                self._logger.error ("Ignore ill-formatted job id (%s) (%s)" % (line, e))
+                self._logger.debug ("Ignore ill-formatted job id (%s) (%s)" % (line, e))
+                continue
+
+            job_id = "[%s]-[%s]" % (self.rm, pid)
+            self._ids.append (job_id)
 
         return self._ids
    
@@ -725,20 +756,13 @@ class ShellJobService (saga.adaptors.cpi.job.Service) :
         """ Implements saga.adaptors.cpi.job.Service.get_url()
         """
 
-        known_jobs = self.list ()
+        # this dict is passed on to the job adaptor class -- use it to pass any
+        # state information you need there.
+        adaptor_state = { "job_service"     : self, 
+                          "job_id"          : jobid,
+                          "job_schema"      : self.rm.schema }
 
-        if jobid not in known_jobs :
-            raise saga.BadParameter._log (self._logger, "job id '%s' unknown"
-                                       % jobid)
-
-        else:
-            # this dict is passed on to the job adaptor class -- use it to pass any
-            # state information you need there.
-            adaptor_state = { "job_service"     : self, 
-                              "job_id"          : jobid,
-                              "job_schema"      : self.rm.schema }
-
-            return saga.job.Job (_adaptor=self._adaptor, _adaptor_state=adaptor_state)
+        return saga.job.Job (_adaptor=self._adaptor, _adaptor_state=adaptor_state)
 
    
     # ----------------------------------------------------------------
@@ -1117,6 +1141,72 @@ class ShellJob (saga.adaptors.cpi.job.Job) :
         return self._finished
 
 
+    # ----------------------------------------------------------------
+    #
+    @SYNC_CALL
+    def get_stdout_string (self) : 
+
+        state = self.get_state () # refresh stats
+
+        if  state == saga.job.NEW      or \
+            state == saga.job.PENDING  :
+            raise saga.IncorrectState ("Job output is only available after the job started")
+
+        if  not self._id :
+            raise saga.IncorrectState ("Job output is only available after the job started")
+
+        rm, pid     = self._adaptor.parse_id (self._id)
+        ret, out, _ = self.js.shell.run_sync ("STDOUT %s\n" % pid)
+
+        if  ret != 0 :
+            raise saga.NoSuccess ("failed to get job stdout for '%s': (%s)(%s)" \
+                               % (self._id, ret, out))
+
+        lines = filter (None, out.split ("\n"))
+
+        if lines[0] != "OK" :
+            raise saga.NoSuccess ("failed to get valid job stdout for '%s' (%s)" % (self._id, lines))
+
+        data = ""
+        for line in lines[1:] :
+            data += "%s\n" % line
+
+        return _decode (data)
+
+
+    # ----------------------------------------------------------------
+    #
+    @SYNC_CALL
+    def get_stderr_string (self) : 
+
+        state = self.get_state () # refresh stats
+
+        if  state == saga.job.NEW      or \
+            state == saga.job.PENDING  :
+            raise saga.IncorrectState ("Job output is only available after the job started")
+
+        if  not self._id :
+            raise saga.IncorrectState ("Job output is only available after the job started")
+
+        rm, pid     = self._adaptor.parse_id (self._id)
+        ret, out, _ = self.js.shell.run_sync ("STDERR %s\n" % pid)
+
+        if  ret != 0 :
+            raise saga.NoSuccess ("failed to get job stderr for '%s': (%s)(%s)" \
+                               % (self._id, ret, out))
+
+        lines = filter (None, out.split ("\n"))
+
+        if lines[0] != "OK" :
+            raise saga.NoSuccess ("failed to get valid job stderr for '%s' (%s)" % (self._id, lines))
+
+        data = ""
+        for line in lines[1:] :
+            data += "%s\n" % line
+
+        return _decode (data)
+
+
     # ----------------------------------------------------------------
     #
     @SYNC_CALL
