@@ -11,6 +11,7 @@ __license__   = "MIT"
 
 import saga.utils.pty_shell
 
+import saga.url as surl
 import saga.adaptors.base
 import saga.adaptors.cpi.job
 
@@ -19,7 +20,6 @@ import os
 import time
 import textwrap
 import string
-from copy import deepcopy
 
 SYNC_CALL  = saga.adaptors.cpi.decorators.SYNC_CALL
 ASYNC_CALL = saga.adaptors.cpi.decorators.ASYNC_CALL
@@ -346,8 +346,8 @@ class SLURMJobService (saga.adaptors.cpi.job.Service) :
         shell_url += self.rm.host
 
         #add port
-        if self.rm.port:
-            shell_url += ":" + self.rm.port
+        if  self.rm.port:
+            shell_url += ":" + str(self.rm.port)
 
         shell_url = saga.url.Url(shell_url)
 
@@ -625,7 +625,7 @@ class SLURMJobService (saga.adaptors.cpi.job.Service) :
     def _slurm_to_saga_jobstate(self, slurmjs):
         """ translates a slurm one-letter state to saga
         """
-        if slurmjs == "CANCELED" or slurmjs == 'CA':
+        if slurmjs == "CANCELLED" or slurmjs == 'CA':
             return saga.job.CANCELED
         elif slurmjs == "COMPLETED" or slurmjs == 'CD':
             return saga.job.DONE
@@ -673,7 +673,7 @@ class SLURMJobService (saga.adaptors.cpi.job.Service) :
                 # if no match, we have no exitcode
                 else:
                     self.exit_code = None
-                    
+
                 self._logger.debug("Returning exit code %s" % self.exit_code)
                 
                 # return whatever our exit code is
@@ -917,8 +917,17 @@ class SLURMJob (saga.adaptors.cpi.job.Job):
             return prev_info
 
         # curr. info will contain the new job info collect. it starts off
-        # as a copy of prev_info
-        curr_info = deepcopy(prev_info)
+        # as a copy of prev_info (don't use deepcopy because there is an API 
+        # object in the dict -> recursion)
+        curr_info = dict()
+        curr_info['job_id'     ] = prev_info.get ('job_id'     )
+        curr_info['state'      ] = prev_info.get ('state'      )
+        curr_info['exec_hosts' ] = prev_info.get ('exec_hosts' )
+        curr_info['returncode' ] = prev_info.get ('returncode' )
+        curr_info['create_time'] = prev_info.get ('create_time')
+        curr_info['start_time' ] = prev_info.get ('start_time' )
+        curr_info['end_time'   ] = prev_info.get ('end_time'   )
+        curr_info['gone'       ] = prev_info.get ('gone'       )
 
         rm, pid = self._adaptor.parse_id(job_id)
 
@@ -1014,12 +1023,17 @@ class SLURMJob (saga.adaptors.cpi.job.Job):
             ret, out, _ = self.js.shell.run_sync('scontrol show job %s' % pid)
             m = self.js.scontrol_jobstate_re.search(out)
             if m:
-                scontrol_state = m.group(0)
+                slurm_state = m.group(0)
             else:
                 # no jobstate found from scontrol
-                return saga.job.UNKNOWN
+                # the job may have finished a while back, use sacct to
+                # look at the full slurm history
+                slurm_state = self._sacct_jobstate_match(pid)
+                if not slurm_state:
+                    # no jobstate found in slurm
+                    return saga.job.UNKNOWN
 
-            return self.js._slurm_to_saga_jobstate(scontrol_state)
+            return self.js._slurm_to_saga_jobstate(slurm_state)
 
         except Exception, ex:
             raise saga.NoSuccess("Error getting the job state for "
@@ -1029,6 +1043,24 @@ class SLURMJob (saga.adaptors.cpi.job.Job):
                                    "Internal SLURM adaptor error"
                                    " in _job_get_state")
 
+    def _sacct_jobstate_match (self, pid):
+        """ get the job state from the slurm accounting data """
+        ret, sacct_out, _ = self.js.shell.run_sync(
+            "sacct --format=JobID,State --parsable2 --noheader --jobs=%s" % pid)
+        # output will look like:
+        # 500723|COMPLETED
+        # 500723.batch|COMPLETED
+        # or:
+        # 500682|CANCELLED by 900369
+        # 500682.batch|CANCELLED
+
+        for line in sacct_out.strip().split('\n'):
+            (slurm_id, slurm_state) = line.split('|', 1)
+            if slurm_id == pid and slurm_state:
+                return slurm_state.split()[0].strip()
+
+        return None
+
     # ----------------------------------------------------------------
     #
     @SYNC_CALL
