@@ -4,14 +4,16 @@ __copyright__ = "Copyright 2012-2013, The SAGA Project"
 __license__   = "MIT"
 
 
-import saga.utils.singleton
-import saga.utils.logger
+import radical.utils            as ru
+import radical.utils.signatures as rus
+import radical.utils.logger     as rul
+
+import saga.exceptions          as se
+
 import saga.engine.engine
 import saga.context
 import saga.base
 
-import saga.exceptions       as se
-import saga.utils.signatures as sus
 
 
 # ------------------------------------------------------------------------------
@@ -36,7 +38,7 @@ class _ContextList (list) :
         if  session : 
             self._logger  = session._logger
         else :
-            self._logger  = saga.utils.logger.getLogger ('ContextList')
+            self._logger  = rul.getLogger ('saga', 'ContextList')
 
         base_list = super  (_ContextList, self)
         base_list.__init__ (*args, **kwargs)
@@ -84,12 +86,12 @@ class _ContextList (list) :
 #
 class _DefaultSession (object) :
 
-    __metaclass__ = saga.utils.singleton.Singleton
+    __metaclass__ = ru.Singleton
 
     # --------------------------------------------------------------------------
     #
-    @sus.takes   ('_DefaultSession')
-    @sus.returns (sus.nothing)
+    @rus.takes   ('_DefaultSession')
+    @rus.returns (rus.nothing)
     def __init__ (self) :
 
         # the default session picks up default contexts, from all context
@@ -97,8 +99,9 @@ class _DefaultSession (object) :
         # dig through the registered context adaptors, and ask each of them for
         # default contexts.
 
-        self.contexts  = _ContextList ()
-        self._logger   = saga.utils.logger.getLogger ('saga.DefaultSession')
+        self.contexts       = _ContextList ()
+        self._lease_manager = ru.LeaseManager ()
+        self._logger        = rul.getLogger ('saga', 'DefaultSession')
 
         _engine = saga.engine.engine.Engine ()
 
@@ -184,9 +187,9 @@ class Session (saga.base.SimpleBase) :
 
     # --------------------------------------------------------------------------
     #
-    @sus.takes   ('Session', 
-                  sus.optional(bool))
-    @sus.returns (sus.nothing)
+    @rus.takes   ('Session', 
+                  rus.optional(bool))
+    @rus.returns (rus.nothing)
     def __init__ (self, default=True) :
         """
         default: bool
@@ -200,17 +203,23 @@ class Session (saga.base.SimpleBase) :
         # shared list of the default session singleton.  Otherwise, we create
         # a private list which is not populated.
 
+        # a session also has a lease manager, for adaptors in this session to use.
+
         if  default :
-            default_session  = _DefaultSession ()
-            self.contexts    = default_session.contexts 
+            default_session     = _DefaultSession ()
+            self.contexts       = default_session.contexts 
+            self._lease_manager = default_session._lease_manager
         else :
-            self.contexts    = _ContextList (session=self)
+            self.contexts       = _ContextList (session=self)
+            self._lease_manager = ru.LeaseManager ()
+
+
 
 
     # ----------------------------------------------------------------
     #
-    @sus.takes   ('Session')
-    @sus.returns (basestring)
+    @rus.takes   ('Session')
+    @rus.returns (basestring)
     def __str__  (self):
         """String represenation."""
 
@@ -219,9 +228,9 @@ class Session (saga.base.SimpleBase) :
 
     # ----------------------------------------------------------------
     #
-    @sus.takes      ('Session', 
+    @rus.takes      ('Session', 
                      saga.context.Context)
-    @sus.returns    (sus.nothing)
+    @rus.returns    (rus.nothing)
     def add_context (self, ctx) :
         """
         ctx:     saga.Context
@@ -236,9 +245,9 @@ class Session (saga.base.SimpleBase) :
 
     # ----------------------------------------------------------------
     #
-    @sus.takes   ('Session', 
+    @rus.takes   ('Session', 
                   saga.context.Context)
-    @sus.returns (sus.nothing)
+    @rus.returns (rus.nothing)
     def remove_context (self, ctx) :
         """
         ctx:     saga.Context
@@ -254,8 +263,8 @@ class Session (saga.base.SimpleBase) :
 
     # ----------------------------------------------------------------
     #
-    @sus.takes   ('Session')
-    @sus.returns (sus.list_of (saga.context.Context))
+    @rus.takes   ('Session')
+    @rus.returns (rus.list_of (saga.context.Context))
     def list_contexts  (self) :
         """
         ret:     list[saga.Context]
@@ -267,6 +276,20 @@ class Session (saga.base.SimpleBase) :
         return self.contexts
 
 
+    # ----------------------------------------------------------------
+    #
+    @rus.takes   ('Session')
+    @rus.returns (dict)
+    def get_config (self, section="") :
+        """
+        ret:     radical.utils.Configuration
+        
+        Return the session configuration (optional a specific section).
+        """
+
+        return saga.engine.engine.Engine ().get_config (section)
+
+
+
 
-# vim: tabstop=8 expandtab shiftwidth=4 softtabstop=4
 
