@@ -153,6 +153,20 @@ class PTYProcess (object) :
                 pass
     
 
+    # ----------------------------------------------------------------------
+    #
+    def _hide_data (self, data, nolog=False) :
+
+        if  nolog :
+            import re
+            return re.sub (r'([^\n])', 'X', data)
+
+        else :
+            return data
+
+
+
+
     # ----------------------------------------------------------------------
     #
     def initialize (self) :
@@ -459,10 +473,18 @@ class PTYProcess (object) :
                 # Boooh!
                 return "false alarm, process %s is alive!" % self.child
 
+            # try a last read to grab whatever we can get (from cache)
+            data = ''
+            try :
+                data  = self.tail
+                data += self.read (size=0, timeout=-1)
+            except :
+                pass
+
             ret  = ""
             ret += "  exit code  : %s\n" % self.exit_code
             ret += "  exit signal: %s\n" % self.exit_signal
-            ret += "  last output: %s\n" % self.cache[-256:] # FIXME: smarter selection
+            ret += "  last output: %s\n" % data
 
             return ret
 
@@ -494,9 +516,6 @@ class PTYProcess (object) :
 
             found_eof = False
 
-            if  not self.alive (recover=False) :
-                raise se.NoSuccess ("process I/O failed: %s" % self.tail)
-
             try:
                 # start the timeout timer right now.  Note that even if timeout is
                 # short, and child.poll is slow, we will nevertheless attempt at least
@@ -676,24 +695,21 @@ class PTYProcess (object) :
                 # a pattern, or timeout passes
                 while True :
 
-
-
-                  # time.sleep (0.1)
-
                     # skip non-lines
                     if  not data :
                         data += self.read (timeout=_POLLDELAY)
                     if  _debug : print ">>%s<<" % data
 
+                    escaped = escape (data)
+                    if _debug : print 'data    ==%s==' % data
+                    if _debug : print 'escaped ==%s==' % escaped
+
                     # check current data for any matching pattern
                     for n in range (0, len(patts)) :
 
-                        escaped = escape (data)
-                      # print '-- 1 --%s--' % data
-                      # print '-- 2 --%s--' % escaped
-
                         match = patts[n].search (escaped)
                         if _debug : print "==%s==" % patterns[n]
+                        if _debug : print match
 
                         if match :
                             # a pattern matched the current data: return a tuple of
@@ -729,7 +745,7 @@ class PTYProcess (object) :
 
     # ----------------------------------------------------------------
     #
-    def write (self, data) :
+    def write (self, data, nolog=False) :
         """
         This method will repeatedly attempt to push the given data into the
         child's stdin pipe, until it succeeds to write all data.
@@ -743,7 +759,8 @@ class PTYProcess (object) :
 
             try :
 
-                log = data.replace ('\n', '\\n')
+                log = self._hide_data (data, nolog)
+                log =  log.replace ('\n', '\\n')
                 log =  log.replace ('\r', '')
                 if  len(log) > _DEBUG_MAX :
                     self.logger.debug ("write: [%5d] [%5d] (%s ... %s)" \
