@@ -179,8 +179,13 @@ class PTYShell (object) :
     #
     def __init__ (self, url, session=None, logger=None, init=None, opts={}) :
 
-        if  None != logger  : self.logger  = logger
-        else                : self.logger  = rul.getLogger ('saga', 'PTYShell') 
+      # print 'new pty shell to %s' % url
+
+        if   logger  : self.logger  = logger
+        else         : self.logger  = rul.getLogger ('saga', 'PTYShell') 
+
+        if   session : self.session = session
+        else         : self.session = ss.Session (default=True)
 
         self.logger.debug ("PTYShell init %s" % self)
 
@@ -188,11 +193,22 @@ class PTYShell (object) :
         self.init        = init     # call after reconnect
         self.opts        = opts     # options...
         self.latency     = 0.0      # set by factory
+        self.cp_slave    = None     # file copy channel
 
-        self.prompt      = None
-        self.prompt_re   = None
         self.initialized = False
 
+        # get prompt pattern from config
+        self.cfg       = self.session.get_config('saga.utils.pty')
+
+        if  'prompt_pattern' in self.cfg :
+            self.prompt    = self.cfg['prompt_pattern'].get_value ()
+            self.prompt_re = re.compile ("^(.*?)%s" % self.prompt, re.DOTALL)
+        else :
+            self.prompt    = "[\$#%>\]]\s*$"
+            self.prompt_re = re.compile ("^(.*?)%s" % self.prompt, re.DOTALL)
+
+        self.logger.info ("PTY prompt pattern: %s" % self.prompt)
+
         # we need a local dir for file staging caches.  At this point we use
         # $HOME, but should make this configurable (FIXME)
         self.base = os.environ['HOME'] + '/.saga/adaptors/shell/'
@@ -208,7 +224,7 @@ class PTYShell (object) :
 
         
         self.factory    = supsf.PTYShellFactory   ()
-        self.pty_info   = self.factory.initialize (url, session, self.logger)
+        self.pty_info   = self.factory.initialize (self.url, self.session, self.prompt, self.logger)
         self.pty_shell  = self.factory.run_shell  (self.pty_info)
 
         self.initialize ()
@@ -244,8 +260,8 @@ class PTYShell (object) :
                 self.logger.info ("custom  command shell: %s" % command_shell)
 
 
-            self.logger.debug    ("running command shell: %s" % command_shell)
-            self.pty_shell.write (" stty -echo ; %s\n"        % command_shell)
+            self.logger.debug    ("running command shell:         %s"   % command_shell)
+            self.pty_shell.write (" stty -echo ; unset HISTFILE ; %s\n" % command_shell)
 
             # make sure this worked, and that we find the prompt. We use
             # a versatile prompt pattern to account for the custom shell case.
@@ -255,10 +271,11 @@ class PTYShell (object) :
             # a versatile prompt pattern to account for the custom shell case.
             try :
                 # set and register new prompt
-                self.run_async  (" unset PROMPT_COMMAND ; "
-                                     + "PS1='PROMPT-$?->'; "
-                                     + "PS2=''; "
-                                     + "export PS1 PS2 2>&1 >/dev/null\n")
+                self.run_async  ( " unset PROMPT_COMMAND ; "
+                                + " unset HISTFILE ; "
+                                + "PS1='PROMPT-$?->'; "
+                                + "PS2=''; "
+                                + "export PS1 PS2 2>&1 >/dev/null\n")
                 self.set_prompt (new_prompt="PROMPT-(\d+)->$")
 
                 self.logger.debug ("got new shell prompt")
@@ -646,13 +663,20 @@ class PTYShell (object) :
                 stdout = None
                 stderr = None
 
+                if  iomode == None :
+                    iomode =  STDOUT
+
                 if  iomode == IGNORE :
                     pass
 
                 if  iomode == MERGED :
                     stdout =  txt
 
-                if  iomode == SEPARATE :
+                if  iomode == STDOUT :
+                    stdout =  txt
+
+                if  iomode == SEPARATE or \
+                    iomode == STDERR   :
                     stdout =  txt
 
                     self.pty_shell.write (" cat %s\n" % _err)
@@ -665,20 +689,16 @@ class PTYShell (object) :
                                               % command)
 
                     _ret, _stderr = self._eval_prompt (match)
+
                     if  _ret :
                         raise se.IncorrectState ("run_sync failed, no stderr (%s: %s)" \
                                               % (_ret, _stderr))
+
                     stderr =  _stderr
 
-
-                if  iomode == STDOUT :
-                    stdout =  txt
-
                 if  iomode == STDERR :
-                    stderr =  txt
-
-                if  iomode == None :
-                    stdout =  txt
+                    # got stderr in branch above
+                    stdout =  None
 
                 return (ret, stdout, stderr)
 
@@ -712,7 +732,7 @@ class PTYShell (object) :
 
             try :
                 command = command.strip ()
-                self.send ("%s\n" % command)
+                self.send (" %s\n" % command)
 
             except Exception as e :
                 raise ptye.translate_exception (e)
@@ -767,7 +787,7 @@ class PTYShell (object) :
             fhandle.flush  ()
             fhandle.close  ()
 
-            ret = self.factory.run_copy_to (self.pty_info, fname, tgt)
+            ret = self.stage_to_remote (fname, tgt)
 
             os.remove (fname)
 
@@ -792,12 +812,13 @@ class PTYShell (object) :
             # prompt, and updating pwd state on every find_prompt.
 
             # first, write data into a tmp file
-            fname   = self.base + "/staging.%s" % id(self)
+            fname = self.base + "/staging.%s" % id(self)
+            _     = self.stage_from_remote (src, fname)
 
-            _ = self.factory.run_copy_from (self.pty_info, src, fname)
+            os.system ('sync') # WTF?  Why do I need this?
 
             fhandle = open (fname, 'r')
-            out = fhandle.read  ()
+            out     = fhandle.read  ()
             fhandle.close  ()
 
             os.remove (fname)
@@ -827,7 +848,7 @@ class PTYShell (object) :
         # prompt, and updating pwd state on every find_prompt.
 
         try :
-            return self.factory.run_copy_to (self.pty_info, src, tgt, cp_flags)
+            return self.run_copy_to (src, tgt, cp_flags)
 
         except Exception as e :
             raise ptye.translate_exception (e)
@@ -851,11 +872,188 @@ class PTYShell (object) :
         # prompt, and updating pwd state on every find_prompt.
 
         try :
-            return self.factory.run_copy_from (self.pty_info, src, tgt, cp_flags)
+            return self.run_copy_from (src, tgt, cp_flags)
 
         except Exception as e :
             raise ptye.translate_exception (e)
 
+    # --------------------------------------------------------------------------
+    #
+    def run_copy_to (self, src, tgt, cp_flags="") :
+        """ 
+        This initiates a slave copy connection.   Src is interpreted as local
+        path, tgt as path on the remote host.
+
+        Now, this is ugly when over sftp: sftp supports recursive copy, and
+        wildcards, all right -- but for recursive copies, it wants the target
+        dir to exist -- so, we have to check if the local src is a  dir, and if
+        so, we first create the target before the copy.  Worse, for wildcards we
+        have to do a local expansion, and the to do the same for each entry...
+        """
+
+        with self.pty_shell.rlock :
+
+            info = self.pty_info
+            repl = dict ({'src'      : src, 
+                          'tgt'      : tgt, 
+                          'cp_flags' : cp_flags}.items () + info.items ())
+
+            # at this point, we do have a valid, living master
+            s_cmd = info['scripts'][info['type']]['copy_to']    % repl
+            s_in  = info['scripts'][info['type']]['copy_to_in'] % repl
+
+            if not self.cp_slave :
+                self.cp_slave = self.factory.get_cp_slave (s_cmd, info)
+
+            prep = ""
+            if  'sftp' in s_cmd :
+                # prepare target dirs for recursive copy, if needed
+                import glob
+                src_list = glob.glob (src)
+                for s in src_list :
+                    if  os.path.isdir (s) :
+                        prep += "mkdir %s/%s\n" % (tgt, os.path.basename (s))
+
+
+            _      = self.cp_slave.write    ("%s%s\n" % (prep, s_in))
+            _, out = self.cp_slave.find     (['[\$\>\]]\s*$'], -1)
+            _, out = self.cp_slave.find     (['[\$\>\]]\s*$'], 1.0)
+
+            # FIXME: we don't really get exit codes from copy
+            # if  self.cp_slave.exit_code != 0 :
+            #     raise se.NoSuccess._log (info['logger'], "file copy failed: %s" % str(out))
+
+            if 'Invalid flag' in out :
+                raise se.NoSuccess._log (info['logger'], "sftp version not supported (%s)" % str(out))
+
+            if 'No such file or directory' in out :
+                raise se.DoesNotExist._log (info['logger'], "file copy failed: %s" % str(out))
+
+            if 'is not a directory' in out :
+                raise se.BadParameter._log (info['logger'], "File copy failed: %s" % str(out))
+
+            if  'sftp' in s_cmd :
+                if 'not found' in out :
+                    raise se.BadParameter._log (info['logger'], "file copy failed: %s" % out)
+
+
+            # we interpret the first word on the line as name of src file -- we
+            # will return a list of those
+            lines = out.split ('\n')
+            files = []
+
+            for line in lines :
+
+                elems = line.split (' ', 2)
+
+                if  elems :
+
+                    f = elems[0]
+
+                    # remove quotes
+                    if  f :
+
+                        if  f[ 0] in ["'", '"', '`'] : f = f[1:  ]
+                        if  f[-1] in ["'", '"', '`'] : f = f[ :-1]
+
+                    # ignore empty lines
+                    if  f :
+
+                        files.append (f)
+
+            info['logger'].debug ("copy done: %s" % files)
+
+            return files
+
+
+    # --------------------------------------------------------------------------
+    #
+    def run_copy_from (self, src, tgt, cp_flags="") :
+        """ 
+        This initiates a slave copy connection.   Src is interpreted as path on
+        the remote host, tgt as local path.
+
+        We have to do the same mkdir trick as for the run_copy_to, but here we
+        need to expand wildcards on the *remote* side :/
+        """
+
+        with self.pty_shell.rlock :
+
+            info = self.pty_info
+            repl = dict ({'src'      : src, 
+                          'tgt'      : tgt, 
+                          'cp_flags' : cp_flags}.items ()+ info.items ())
+
+            # at this point, we do have a valid, living master
+            s_cmd = info['scripts'][info['type']]['copy_from']    % repl
+            s_in  = info['scripts'][info['type']]['copy_from_in'] % repl
+
+            if  not self.cp_slave :
+                self.cp_slave = self.factory.get_cp_slave (s_cmd, info)
+
+            prep = ""
+
+            if  'sftp' in s_cmd :
+                # prepare target dirs for recursive copy, if needed
+                self.cp_slave.write (" ls %s\n" % src)
+                _, out = self.cp_slave.find (["^sftp> "], -1)
+
+                src_list = out[1].split ('/n')
+
+                for s in src_list :
+                    if  os.path.isdir (s) :
+                        prep += "lmkdir %s/%s\n" % (tgt, os.path.basename (s))
+
+
+            _      = self.cp_slave.write    ("%s%s\n" % (prep, s_in))
+            _, out = self.cp_slave.find     (['[\$\>\]] *$'], -1)
+
+            # FIXME: we don't really get exit codes from copy
+          # if  self.cp_slave.exit_code != 0 :
+          #     raise se.NoSuccess._log (info['logger'], "file copy failed: %s" % out)
+
+            if 'Invalid flag' in out :
+                raise se.NoSuccess._log (info['logger'], "sftp version not supported (%s)" % out)
+
+            if 'No such file or directory' in out :
+                raise se.DoesNotExist._log (info['logger'], "file copy failed: %s" % out)
+
+            if 'is not a directory' in out :
+                raise se.BadParameter._log (info['logger'], "file copy failed: %s" % out)
+
+            if  'sftp' in s_cmd :
+                if 'not found' in out :
+                    raise se.BadParameter._log (info['logger'], "file copy failed: %s" % out)
+
+
+            # we run copy with -v, so get a list of files which have been copied
+            # -- we parse that list and return it.  we interpret the *second*
+            # word on the line as name of src file.
+            lines = out.split ('\n')
+            files = []
+
+            for line in lines :
+
+                elems = line.split (' ', 3)
+                
+                if  elems and len(elems) > 1 and elems[0] == 'Fetching' :
+
+                    f = elems[1]
+
+                    # remove quotes
+                    if  f :
+
+                        if  f[ 0] in ["'", '"', '`']  :  f = f[1:  ]
+                        if  f[-1] in ["'", '"', '`']  :  f = f[ :-1]
+
+                    # ignore empty lines
+                    if  f :
+                        files.append (f)
+
+            info['logger'].debug ("copy done: %s" % files)
+
+            return files
+
 
 
 
