@@ -47,7 +47,7 @@ import pty_exceptions               as ptye
 # these arrays help to map requested client schemas to master schemas
 _SCHEMAS_SH  = ['sh', 'fork', 'local', 'file']
 _SCHEMAS_SSH = ['ssh', 'scp', 'sftp']
-_SCHEMAS_GSI = ['gsissh', 'gsiscp', 'gsisftp', 'gsiftp']
+_SCHEMAS_GSI = ['gsissh', 'gsiscp', 'gsisftp']
 
 _SCHEMAS = _SCHEMAS_SH + _SCHEMAS_SSH + _SCHEMAS_GSI
 
@@ -59,29 +59,29 @@ _SCHEMAS = _SCHEMAS_SH + _SCHEMAS_SSH + _SCHEMAS_GSI
 
 # ssh master/slave flag magic # FIXME: make timeouts configurable
 _SSH_FLAGS_MASTER   = "-o ControlMaster=yes -o ControlPath=%(ctrl)s"
-_SSH_FLAGS_SLAVE    = "-o ControlMaster=no  -o ControlPath=%(ctrl)s"
+_SSH_FLAGS_SLAVE    = "-o ControlMaster=no   -o ControlPath=%(ctrl)s"
 
 # FIXME: right now, we create a shell connection as master --
 # but a master does not actually need a shell, as it is never really
 # used to run commands...
 _SCRIPTS = {
     'ssh' : { 
-        'master'        : "%(ssh_env)s %(ssh_exe)s   %(ssh_args)s  %(m_flags)s  %(host_str)s",
-        'shell'         : "%(ssh_env)s %(ssh_exe)s   %(ssh_args)s  %(s_flags)s  %(host_str)s",
-      # 'copy_to'       : "%(scp_env)s %(scp_exe)s   %(scp_args)s  %(s_flags)s  %(src)s %(root)s/%(tgt)s",
-      # 'copy_from'     : "%(scp_env)s %(scp_exe)s   %(scp_args)s  %(s_flags)s  %(root)s/%(src)s %(tgt)s",
-        'copy_to'       : "%(sftp_env)s %(sftp_exe)s %(sftp_args)s %(s_flags)s  %(host_str)s",
-        'copy_from'     : "%(sftp_env)s %(sftp_exe)s %(sftp_args)s %(s_flags)s  %(host_str)s",
-        'copy_to_in'    : "mput %(cp_flags)s %(src)s %(tgt)s \n",
-        'copy_from_in'  : "mget %(cp_flags)s %(src)s %(tgt)s \n",
+        'master'        : '%(ssh_env)s "%(ssh_exe)s"   %(ssh_args)s  %(m_flags)s  %(host_str)s',
+        'shell'         : '%(ssh_env)s "%(ssh_exe)s"   %(ssh_args)s  %(s_flags)s  %(host_str)s',
+      # 'copy_to'       : '%(scp_env)s "%(scp_exe)s"   %(scp_args)s  %(s_flags)s  "%(src)s" "%(fs_root)s/%(tgt)s"',
+      # 'copy_from'     : '%(scp_env)s "%(scp_exe)s"   %(scp_args)s  %(s_flags)s  "%(fs_root)s/%(src)s" "%(tgt)s"',
+        'copy_to'       : '%(sftp_env)s "%(sftp_exe)s" %(sftp_args)s %(s_flags)s  %(host_str)s',
+        'copy_from'     : '%(sftp_env)s "%(sftp_exe)s" %(sftp_args)s %(s_flags)s  %(host_str)s',
+        'copy_to_in'    : 'mput %(cp_flags)s "%(src)s" "%(tgt)s" \n',
+        'copy_from_in'  : 'mget %(cp_flags)s "%(src)s" "%(tgt)s" \n',
     },
-    'sh' : { 
-        'master'        : "%(sh_env)s %(sh_exe)s  %(sh_args)s",
-        'shell'         : "%(sh_env)s %(sh_exe)s  %(sh_args)s",
-        'copy_to'       : "%(sh_env)s %(sh_exe)s  %(sh_args)s",
-        'copy_from'     : "%(sh_env)s %(sh_exe)s  %(sh_args)s",
-        'copy_to_in'    : "cd ~ && %(cp_exe)s -v %(cp_flags)s '%(src)s' '%(tgt)s'",
-        'copy_from_in'  : "cd ~ && %(cp_exe)s -v %(cp_flags)s '%(src)s' '%(tgt)s'",
+    'sh' : {
+        'master'        : '%(sh_env)s "%(sh_exe)s"  %(sh_args)s',
+        'shell'         : '%(sh_env)s "%(sh_exe)s"  %(sh_args)s',
+        'copy_to'       : '%(sh_env)s "%(sh_exe)s"  %(sh_args)s',
+        'copy_from'     : '%(sh_env)s "%(sh_exe)s"  %(sh_args)s',
+        'copy_to_in'    : 'cd ~ && "%(cp_exe)s" -v %(cp_flags)s "%(src)s" "%(tgt)s"',
+        'copy_from_in'  : 'cd ~ && "%(cp_exe)s" -v %(cp_flags)s "%(src)s" "%(tgt)s"',
     }
 }
 
@@ -136,24 +136,26 @@ class PTYShellFactory (object) :
         self.logger     = rul.getLogger ('saga', 'PTYShellFactory')
         self.registry   = {}
         self.rlock      = ru.RLock ('pty shell factory')
-        self._cp_slaves = dict()
 
 
     # --------------------------------------------------------------------------
     #
-    def initialize (self, url, session=None, logger=None) :
+    def initialize (self, url, session=None, prompt=None, logger=None) :
 
         with self.rlock :
 
             # make sure we have a valid url type
             url = saga.Url (url)
 
+            if  not prompt :
+                prompt = "^(.*[\$#%>\]])\s*$"
+
             if  not logger :
                 logger = rul.getLogger ('saga', 'PTYShellFactory')
 
             # collect all information we have/need about the requested master
             # connection
-            info = self._create_master_entry (url, session, logger)
+            info = self._create_master_entry (url, session, prompt, logger)
 
             # we got master info - register the master, and create the instance!
             type_s = str(info['type'])
@@ -166,7 +168,7 @@ class PTYShellFactory (object) :
             if not type_s in self.registry[host_s][user_s] :
 
                 # new master: create an instance, and register it
-                m_cmd = _SCRIPTS[info['type']]['master'] % info
+                m_cmd = info['scripts'][info['type']]['master'] % info
 
                 logger.debug ("open master pty for [%s] [%s] %s: %s'" \
                                 % (type_s, host_s, user_s, m_cmd))
@@ -174,7 +176,7 @@ class PTYShellFactory (object) :
                 info['pty'] = supp.PTYProcess (m_cmd, logger=logger)
                 if not info['pty'].alive () :
                     raise se.NoSuccess._log (logger, \
-                	  "Shell not connected to %s" % info['host_str'])
+                          "Shell not connected to %s" % info['host_str'])
 
                 # authorization, prompt setup, etc
                 self._initialize_pty (info['pty'], info, is_shell=True)
@@ -190,7 +192,7 @@ class PTYShellFactory (object) :
 
                 if  not info['pty'].alive (recover=True) :
                     raise se.IncorrectState._log (logger, \
-                	  "Lost shell connection to %s" % info['host_str'])
+                          "Lost shell connection to %s" % info['host_str'])
 
             return info
 
@@ -206,6 +208,7 @@ class PTYShellFactory (object) :
 
             shell_pass = info['pass']
             key_pass   = info['key_pass']
+            prompt     = info['prompt']
             logger     = info['logger']
             latency    = info['latency']
 
@@ -224,11 +227,12 @@ class PTYShellFactory (object) :
                                    "Token_Response.*:\s*$",        # passtoken  prompt
                                    "want to continue connecting",  # hostkey confirmation
                                    ".*HELLO_\\d+_SAGA$",           # prompt detection helper
-                                   "^(.*[\$#%>\]])\s*$"]           # greedy native shell prompt 
+                                   prompt]                         # greedy native shell prompt 
 
                 # find a prompt
-                # use a very aggressive, but portable prompt setting scheme
-              # pty_shell.write (" export PS1='$' > /dev/null 2>&1 || set prompt='$'\n")
+                # use a very aggressive, but portable prompt setting scheme.
+                # Error messages may appear for tcsh and others
+                pty_shell.write (" export PS1='$' ; set prompt='$'\n")
                 n, match = pty_shell.find (prompt_patterns, delay)
 
                 # this loop will run until we finally find the shell prompt, or
@@ -257,13 +261,14 @@ class PTYShellFactory (object) :
                         if  retries > 100 :
                             raise se.NoSuccess ("Could not detect shell prompt (timeout)")
 
+                        # make sure we retry a finite time...
+                        retries += 1
+
                         if  not retry_trigger : 
                             # just waiting for the *right* trigger or prompt, 
                             # don't need new ones...
                             continue
 
-                        retries += 1
-
                         if  is_shell :
                             # use a very aggressive, but portable prompt setting scheme
                             pty_shell.write (" export PS1='$' > /dev/null 2>&1 || set prompt='$'\n")
@@ -281,7 +286,7 @@ class PTYShellFactory (object) :
                             raise se.AuthenticationFailed ("prompted for unknown password (%s)" \
                                                           % match)
 
-                        pty_shell.write ("%s\n" % shell_pass)
+                        pty_shell.write ("%s\n" % shell_pass, nolog=True)
                         n, match = pty_shell.find (prompt_patterns, delay)
 
 
@@ -301,7 +306,7 @@ class PTYShellFactory (object) :
                             raise se.AuthenticationFailed ("prompted for unknown key password (%s)" \
                                                           % key)
 
-                        pty_shell.write ("%s\n" % key_pass[key])
+                        pty_shell.write ("%s\n" % key_pass[key], nolog=True)
                         n, match = pty_shell.find (prompt_patterns, delay)
 
 
@@ -310,7 +315,7 @@ class PTYShellFactory (object) :
                         logger.info ("got token prompt")
                         import getpass
                         token = getpass.getpass ("enter token: ")
-                        pty_shell.write ("%s\n" % token.strip())
+                        pty_shell.write ("%s\n" % token.strip(), nolog=True)
                         n, match = pty_shell.find (prompt_patterns, delay)
 
 
@@ -370,7 +375,7 @@ class PTYShellFactory (object) :
                                     continue
 
 
-                        logger.info ("Got initial shell prompt (%s) (%s)" \
+                        logger.debug ("Got initial shell prompt (%s) (%s)" \
                                    % (n, match))
                         # we are done waiting for a prompt
                         break
@@ -381,20 +386,16 @@ class PTYShellFactory (object) :
 
     # --------------------------------------------------------------------------
     #
-    def _get_cp_slave (self, s_cmd, info) :
+    def get_cp_slave (self, s_cmd, info) :
 
         with self.rlock :
 
-            host = info.get ('host_str', 'any')
+          # print 'new cp  shell to %s' % s_cmd
 
-            if  host in self._cp_slaves :
-                if  self._cp_slaves[host].alive () :
-                    return self._cp_slaves[host]
-            
-            self._cp_slaves[host] = supp.PTYProcess (s_cmd, info['logger'])
-            self._initialize_pty (self._cp_slaves[host], info)
+            cp_slave = supp.PTYProcess (s_cmd, info['logger'])
+            self._initialize_pty (cp_slave, info)
 
-            return self._cp_slaves[host]
+            return cp_slave
 
     # --------------------------------------------------------------------------
     #
@@ -408,7 +409,7 @@ class PTYShellFactory (object) :
       # if True :
         with self.rlock :
 
-            s_cmd = _SCRIPTS[info['type']]['shell'] % info
+            s_cmd = info['scripts'][info['type']]['shell'] % info
 
             # at this point, we do have a valid, living master
             sh_slave = supp.PTYProcess (s_cmd, info['logger'])
@@ -421,181 +422,7 @@ class PTYShellFactory (object) :
 
     # --------------------------------------------------------------------------
     #
-    def run_copy_to (self, info, src, tgt, cp_flags="") :
-        """ 
-        This initiates a slave copy connection.   Src is interpreted as local
-        path, tgt as path on the remote host.
-
-        Now, this is ugly when over sftp: sftp supports recursive copy, and
-        wildcards, all right -- but for recursive copies, it wants the target
-        dir to exist -- so, we have to check if the local src is a  dir, and if
-        so, we first create the target before the copy.  Worse, for wildcards we
-        have to do a local expansion, and the to do the same for each entry...
-        """
-
-        with self.rlock :
-
-            repl = dict ({'src'      : src, 
-                          'tgt'      : tgt, 
-                          'cp_flags' : cp_flags}.items () + info.items ())
-
-            # at this point, we do have a valid, living master
-            s_cmd = _SCRIPTS[info['type']]['copy_to']    % repl
-            s_in  = _SCRIPTS[info['type']]['copy_to_in'] % repl
-
-            cp_slave = self._get_cp_slave (s_cmd, info)
-
-            prep = ""
-            if  'sftp' in s_cmd :
-                # prepare target dirs for recursive copy, if needed
-                import glob
-                src_list = glob.glob (src)
-                for s in src_list :
-                    if  os.path.isdir (s) :
-                        prep += "mkdir %s/%s\n" % (tgt, os.path.basename (s))
-
-
-            _      = cp_slave.write    ("%s%s\n" % (prep, s_in))
-            _, out = cp_slave.find     (['[\$\>\]]\s*$'], -1)
-            _, out = cp_slave.find     (['[\$\>\]]\s*$'], 1.0)
-
-            # FIXME: we don't really get exit codes from copy
-            # if  cp_slave.exit_code != 0 :
-            #     raise se.NoSuccess._log (info['logger'], "file copy failed: %s" % str(out))
-
-            if 'Invalid flag' in out :
-                raise se.NoSuccess._log (info['logger'], "sftp version not supported (%s)" % str(out))
-
-            if 'No such file or directory' in out :
-                raise se.DoesNotExist._log (info['logger'], "file copy failed: %s" % str(out))
-
-            if 'is not a directory' in out :
-                raise se.BadParameter._log (info['logger'], "File copy failed: %s" % str(out))
-
-            if  'sftp' in s_cmd :
-                if 'not found' in out :
-                    raise se.BadParameter._log (info['logger'], "file copy failed: %s" % out)
-
-
-            # we interpret the first word on the line as name of src file -- we
-            # will return a list of those
-            lines = out.split ('\n')
-            files = []
-
-            for line in lines :
-
-                elems = line.split (' ', 2)
-
-                if  elems :
-
-                    f = elems[0]
-
-                    # remove quotes
-                    if  f :
-
-                        if  f[ 0] in ["'", '"', '`'] : f = f[1:  ]
-                        if  f[-1] in ["'", '"', '`'] : f = f[ :-1]
-
-                    # ignore empty lines
-                    if  f :
-
-                        files.append (f)
-
-            info['logger'].debug ("copy done: %s" % files)
-
-            return files
-
-
-    # --------------------------------------------------------------------------
-    #
-    def run_copy_from (self, info, src, tgt, cp_flags="") :
-        """ 
-        This initiates a slave copy connection.   Src is interpreted as path on
-        the remote host, tgt as local path.
-
-        We have to do the same mkdir trick as for the run_copy_to, but here we
-        need to expand wildcards on the *remote* side :/
-        """
-
-        with self.rlock :
-
-            repl = dict ({'src'      : src, 
-                          'tgt'      : tgt, 
-                          'cp_flags' : cp_flags}.items ()+ info.items ())
-
-            # at this point, we do have a valid, living master
-            s_cmd = _SCRIPTS[info['type']]['copy_from']    % repl
-            s_in  = _SCRIPTS[info['type']]['copy_from_in'] % repl
-
-            cp_slave = self._get_cp_slave (s_cmd, info)
-
-            prep = ""
-
-            if  'sftp' in s_cmd :
-                # prepare target dirs for recursive copy, if needed
-                cp_slave.write (" ls %s\n" % src)
-                _, out = cp_slave.find (["^sftp> "], -1)
-
-                src_list = out[1].split ('/n')
-
-                for s in src_list :
-                    if  os.path.isdir (s) :
-                        prep += "lmkdir %s/%s\n" % (tgt, os.path.basename (s))
-
-
-            _      = cp_slave.write    ("%s%s\n" % (prep, s_in))
-            _, out = cp_slave.find     (['[\$\>\]] *$'], -1)
-
-            # FIXME: we don't really get exit codes from copy
-            # if  cp_slave.exit_code != 0 :
-            #     raise se.NoSuccess._log (info['logger'], "file copy failed: %s" % cp_slave.cache[-256:])
-
-            if 'Invalid flag' in out :
-                raise se.NoSuccess._log (info['logger'], "sftp version not supported (%s)" % out)
-
-            if 'No such file or directory' in out :
-                raise se.DoesNotExist._log (info['logger'], "file copy failed: %s" % out)
-
-            if 'is not a directory' in out :
-                raise se.BadParameter._log (info['logger'], "file copy failed: %s" % out)
-
-            if  'sftp' in s_cmd :
-                if 'not found' in out :
-                    raise se.BadParameter._log (info['logger'], "file copy failed: %s" % out)
-
-
-            # we run copy with -v, so get a list of files which have been copied
-            # -- we parse that list and return it.  we interpret the *second*
-            # word on the line as name of src file.
-            lines = out.split ('\n')
-            files = []
-
-            for line in lines :
-
-                elems = line.split (' ', 3)
-                
-                if  elems and len(elems) > 1 and elems[0] == 'Fetching' :
-
-                    f = elems[1]
-
-                    # remove quotes
-                    if  f :
-
-                        if  f[ 0] in ["'", '"', '`']  :  f = f[1:  ]
-                        if  f[-1] in ["'", '"', '`']  :  f = f[ :-1]
-
-                    # ignore empty lines
-                    if  f :
-                        files.append (f)
-
-            info['logger'].debug ("copy done: %s" % files)
-
-            return files
-
-
-    # --------------------------------------------------------------------------
-    #
-    def _create_master_entry (self, url, session, logger) :
+    def _create_master_entry (self, url, session, prompt, logger) :
         # FIXME: cache 'which' results, etc
         # FIXME: check 'which' results
 
@@ -606,10 +433,12 @@ class PTYShellFactory (object) :
 
             info['schema']    = url.schema.lower ()
             info['host_str']  = url.host
+            info['prompt']    = prompt
             info['logger']    = logger
             info['url']       = url
             info['pass']      = ""
             info['key_pass']  = {}
+            info['scripts']   = _SCRIPTS
 
             if  not info['schema'] :
                 info['schema'] = 'local'
@@ -644,7 +473,7 @@ class PTYShellFactory (object) :
 
             else :
                 raise se.BadParameter._log (self.logger, \
-                	  "cannot handle schema '%s://'" % url.schema)
+                          "cannot handle schema '%s://'" % url.schema)
 
 
             # depending on type, create command line (args, env etc)
@@ -754,10 +583,10 @@ class PTYShellFactory (object) :
 
                 if  'user' in info and info['user'] :
                     info['host_str'] = "%s@%s"  % (info['user'], info['host_str'])
-                    info['ctrl'] = "%s_%%h_%%p.%s.%s.ctrl" % (ctrl_base, os.getpid (), info['user'])
+                    info['ctrl'] = "%s_%%h_%%p.%s.ctrl" % (ctrl_base, info['user'])
                 else :
                     info['user'] = getpass.getuser ()
-                    info['ctrl'] = "%s_%%h_%%p.%s.ctrl" % (ctrl_base, os.getpid ())
+                    info['ctrl'] = "%s_%%h_%%p.ctrl" % (ctrl_base)
 
                 info['m_flags']  = _SSH_FLAGS_MASTER % ({'ctrl' : info['ctrl']})
                 info['s_flags']  = _SSH_FLAGS_SLAVE  % ({'ctrl' : info['ctrl']})
