@@ -34,7 +34,7 @@ def _silent_close_js(js_obj):
 # ------------------------------------------------------------------------------
 #
 def test_job_service_get_url():
-    """ Test if the job service URL is returned cotrrectly
+    """ Test if the job service URL is returned correctly
     """
     js = None
     try:
